@@ -74,10 +74,10 @@ class Type(ABC):
             ))
 
     def __lt__(self, other: Type) -> Optional[bool]:
-        return self != other and self <= other
+        return not self.unifiable(other) and self <= other
 
     def __gt__(self, other: Type) -> Optional[bool]:
-        return self != other and self >= other
+        return not self.unifiable(other) and self >= other
 
     def __le__(self, other: Type) -> Optional[bool]:
         return self.instance().unifiable(other.instance(),
@@ -229,28 +229,20 @@ class TypeInstance(Type):
     def __contains__(self, value: TypeInstance) -> bool:
         a = self.follow()
         b = value.follow()
-        return a == b or (
+        return a.unifiable(b) is True or (
             isinstance(a, TypeOperation) and
             any(b in t for t in a.params))
 
-    def variables(self) -> List[TypeVar]:
+    def variables(self) -> Set[TypeVar]:
         """
         Obtain all distinct type variables currently in the type expression,
         including variables that might occur in constraints.
         """
-        # TODO a better way is needed - I'm not sure that id's will remain
-        # distinct in all situations
-        captured = set()
-        result = []
-        for v in self.variables_iter():
-            if v.id not in captured:
-                result.append(v)
-                captured.add(v.id)
+        result = set(self.variables_iter())
+        for v in list(result):
             for c in v.constraints:
                 for v1 in c.variables_iter():
-                    if v1.id not in captured:
-                        result.append(v1)
-                        captured.add(v1.id)
+                    result.add(v1)
         return result
 
     def variables_iter(self) -> Iterable[TypeVar]:
@@ -293,8 +285,9 @@ class TypeInstance(Type):
             subtype: bool = False,
             accept_wildcard: bool = False) -> Optional[bool]:
         """
-        Return True if self can definitely be (a subtype of) other, False if it
-        is definitely not, and None if there is not enough information.
+        Return True if self is definitely the same as (or a subtype of) other,
+        False if it is definitely not, and None if there is not enough
+        information.
         """
         a = self.follow()
         b = other.follow()
@@ -426,12 +419,7 @@ class TypeOperation(TypeInstance):
             return str(self.operator)
 
     def __eq__(self, other: object) -> bool:
-        if isinstance(other, TypeVar):
-            other = other.follow()
-        if isinstance(other, TypeOperation):
-            return (self.operator == other.operator and
-                all(s == t for s, t in zip(self.params, other.params)))
-        return False
+        return isinstance(other, TypeInstance) and bool(self.unifiable(other))
 
     @property
     def basic(self) -> bool:
@@ -442,30 +430,27 @@ class TypeVar(TypeInstance):
     """
     A type variable. This is not a schematic variable — it is instantiated!
     """
-    counter = 0
 
     def __init__(self, name: Optional[str] = None, wildcard: bool = False):
-        self.name = name
+        self._name = name
         self.wildcard = wildcard
-        self.lower: Optional[TypeOperator] = None
         self.unified: Optional[TypeInstance] = None
+        self.lower: Optional[TypeOperator] = None
         self.upper: Optional[TypeOperator] = None
         self.constraints: Set[Constraint] = set()
-        cls = type(self)
-        self.id = cls.counter
-        cls.counter += 1
 
     def __str__(self) -> str:
         if self.unified:
             return str(self.unified)
-        return "_" if self.wildcard else self.name or f"_{self.id}"
+        return "_" if self.wildcard else self.name
 
-    def __eq__(self, other: object) -> bool:
-        if self.unified:
-            return self.follow() == other
-        elif isinstance(other, TypeVar) and other.unified:
-            return self == other.follow()
-        return super().__eq__(other)
+    @property
+    def name(self) -> str:
+        return self._name or f"var{hash(self)}"
+
+    @name.setter
+    def name(self, value: str) -> None:
+        self._name = value
 
     def bind(self, t: TypeInstance) -> None:
         assert (not self.unified or t == self.unified), \
@@ -576,10 +561,10 @@ class Constraint(object):
     def set_context(self, context: TypeInstance) -> None:
         self.description = f"{context} | {self}"
 
-    def variables(self) -> List[TypeVar]:
+    def variables(self) -> Set[TypeVar]:
         result = self.reference.variables()
         for t in self.alternatives:
-            result.extend(t.variables())
+            result = result.union(t.variables())
         return result
 
     def variables_iter(self) -> Iterable[TypeVar]:
